# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2024 Tomas Votava <thomasvotava@seznam.cz>

import json
import http.client
import time
import argparse
from pathlib import Path


FLATHUB_URL = "flathub.org"
JSON_FLATHUB_PATH = "/api/v2/quality-moderation/" "passing-apps?page={0}&page_size=25"
LOCAL_FLATHUB_FILE = "../data/flathub-apps.txt"


GNOME_URL = "gitlab.gnome.org"
JSON_GNOME_PATH = "/Teams/Circle/-/raw/main/data/apps.json"
LOCAL_GNOME_FILE = "../data/gnome-apps.txt"

VALID_ARGUMENTS: set[str] = {"gnome", "flathub"}

# List of categories taken from:
# https://specifications.freedesktop.org/menu-spec/latest/apa.html
# on 27. 06. 2024
VALID_CATEGORIES: set[str] = {
    "Audio",
    "Video",
    "AudioVideo",
    "Development",
    "Education",
    "Game",
    "Graphics",
    "Network",
    "Office",
    "Science",
    "Settings",
    "System",
    "Utility",
    #   Custom categories
    "Featured",
}

VALID_FLAGS: set[str] = {"popular", "skip", "featured", ""}

VALID_DEFAULTS: set[str] = {"default-flags", "default-categories"}


def check_valid_values(valid_set: set[str], check_list: list[str]) -> bool:
    if len(check_list) == 0 or (len(check_list) == 1 and check_list[0] == ""):
        return True
    for string in check_list:
        if string not in valid_set:
            print("{0} is not valid option".format(string))
            return False
    return True


def get_flathub_ids() -> list[str] | None:
    flathub_IDs: list[str] = []
    index: int = 1
    connection = http.client.HTTPSConnection(FLATHUB_URL)
    #    Emulation of do while cycle
    while True:
        connection.request("GET", JSON_FLATHUB_PATH.format(index))
        response = connection.getresponse()
        if response.reason != "OK":
            connection.close()
            return None
        unprocessed_data = response.read().decode("utf-8")
        data_in = json.loads(unprocessed_data)
        flathub_IDs.extend(data_in["apps"])
        if data_in["pagination"]["total_pages"] == index:
            break
        index += 1
    connection.close()
    return flathub_IDs


def save_IDs(
    content: list[str], timestamp: time.struct_time, ids_file_location: str
) -> None:
    options: list[tuple[str, str]] = []
    env: dict[str, list[str]] = {}
    old_file = Path(ids_file_location)
    content.sort()
    if old_file.is_file():
        options, env = read_existing_IDs(ids_file_location)
    output: str = ""
    output += "# Generated on " + time.asctime(timestamp) + "\n"
    output += """# SPDX-License-Identifier: LGPL-2.1-or-later

"""
    for option, value in options:
        output += option + "=" + value + "\n"
    for id in content:
        out = id
        additional_data = env.get(id, [])
        for data in additional_data:
            out += "\t" + data
        output += out + "\n"
    with open(ids_file_location, "w") as file:
        file.write(output)
        return


def check_options(options_list: list[tuple[str, str]], option: str) -> bool:
    for active_option, _ in options_list:
        if active_option == option:
            return False
    return True


def read_existing_IDs(
    ids_file_location: str,
) -> tuple[list[tuple[str, str]], dict[str, list[str]]]:
    enviroment: dict[str, list[str]] = {}
    options: list[tuple[str, str]] = []
    with open(ids_file_location, "r") as file:
        readable_line_index = 0
        default_values = True
        line_index = 0
        for line in file:
            line_index += 1
            line = line.strip()
            if len(line) == 0 or line[0] == "#":
                continue
            readable_line_index += 1
            parameters = line.split("\t")
            if len(parameters) > 3:
                raise ValueError(
                    "Invalid value in line {0}: {1}\n{2}".format(
                        line_index, line, "Too many columns"
                    )
                )
            if len(parameters) == 1:
                parameters = parameters[0].split("=")
                if len(parameters) == 1:
                    if check_options(options, "default-flags"):
                        raise ValueError(
                            "Invalid value in line {0}: {1}\n{2}".format(
                                line_index, line, "No default flags used"
                            )
                        )
                    enviroment[parameters[0]] = []
                    default_values = False
                    continue
                if len(parameters) != 2:
                    raise ValueError(
                        "Invalid value in line {0}: {1}\n{2}".format(
                            line_index, line, "Invalid format of default-flags"
                        )
                    )
                if (
                    not default_values
                    or readable_line_index > len(VALID_DEFAULTS)
                    or not check_options(options, parameters[0])
                ):
                    raise ValueError(
                        "Invalid value in line {0}: {1}\n{2}".format(
                            line_index,
                            line,
                            "Too many default options or default "
                            "options are not at the start of the file",
                        )
                    )
                if not check_valid_values(VALID_DEFAULTS, [parameters[0]]):
                    raise ValueError(
                        "Invalid value in line {0}: {1}\n{2}".format(
                            line_index, line, "Invalid default option"
                        )
                    )
                if parameters[0] == "default-flags" and not check_valid_values(
                    VALID_FLAGS, parameters[1].split(",")
                ):
                    raise ValueError(
                        "Invalid value in line {0}: {1}\n{2}".format(
                            line_index, line, "Invalid Flag"
                        )
                    )
                if parameters[0] == "default-categories" and not check_valid_values(
                    VALID_CATEGORIES, parameters[1].split(",")
                ):
                    raise ValueError(
                        "Invalid value in line {0}: {1}\n{2}".format(
                            line_index, line, "Invalid category"
                        )
                    )
                options.append((parameters[0], parameters[1]))
                continue
            default_values = False
            enviroment[parameters[0]] = parameters
            if parameters[1] == "" and check_options(options, "default-flags"):
                raise ValueError(
                    "Invalid value in line {0}: {1}\n{2}".format(
                        line_index, line, "No default flags used"
                    )
                )
            parameters.pop(0)
            if len(parameters) > 0 and not check_valid_values(
                VALID_FLAGS, parameters[0].split(",")
            ):
                raise ValueError(
                    "Invalid value in line {0}: {1}\n{2}".format(
                        line_index, line, "Invalid Flag"
                    )
                )
            if len(parameters) > 1 and not check_valid_values(
                VALID_CATEGORIES, parameters[1].split(",")
            ):
                raise ValueError(
                    "Invalid value in line {0}: {1}\n{2}".format(
                        line_index, line, "Invalid Flag"
                    )
                )
    return (options, enviroment)


def get_gnome_ids() -> list[str] | None:
    gnome_IDs: list[str] = []
    connection = http.client.HTTPSConnection(GNOME_URL)
    connection.request("GET", JSON_GNOME_PATH)
    response = connection.getresponse()
    if response.reason != "OK":
        connection.close()
        return None
    unprocessed_data = response.read().decode("utf-8")
    data_in = json.loads(unprocessed_data)
    for component in data_in:
        gnome_IDs.append(component["app_id"])
    connection.close()
    return gnome_IDs


def update_gnome() -> None:
    new_ids = get_gnome_ids()
    if new_ids is None:
        print("Couldn't load new GNOME apps")
        return
    save_IDs(new_ids, time.gmtime(), LOCAL_GNOME_FILE)


def update_flathub() -> None:
    new_ids = get_flathub_ids()
    if new_ids is None:
        print("Couldn't load new Flathub apps")
        return
    save_IDs(new_ids, time.gmtime(), LOCAL_FLATHUB_FILE)


UPDATED_STRING = "{0} apps updated\n"


def main() -> int:
    parser = argparse.ArgumentParser(
        description="Updates ID files.",
        epilog="""Choosing no option will update both
    GNOME and Flathub apps list""",
    )
    parser.add_argument(
        "options",
        type=str,
        nargs="*",
        choices=["flathub", "gnome"],
        help="""specifies which files to update, use \'gnome\' to
        update GNOME apps and/or \'flathub\'
        to update Flathub apps""",
    )
    args = parser.parse_args()
    if len(args.options) == 0:
        print("Updating Flathub apps and GNOME apps...")
        update_flathub()
        update_gnome()
        print(UPDATED_STRING.format("Flathub and GNOME"))
        return 0

    if "flathub" in set(args.options):
        print("Updating Flathub apps...")
        update_flathub()
        print(UPDATED_STRING.format("Flathub"))

    if "gnome" in set(args.options):
        print("Updating GNOME apps...")
        update_gnome()
        print(UPDATED_STRING.format("GNOME"))

    return 0


if __name__ == "__main__":
    main()



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2024 Tomas Votava <thomasvotava@seznam.cz>

from pathlib import Path
from update_apps import (
    check_valid_values,
    VALID_CATEGORIES,
    VALID_FLAGS,
    VALID_DEFAULTS,
)

ID_List = dict[str, list[set[str]]]


def read_IDs_file(ids_file_location: str, main_ID_dict: ID_List) -> ID_List:
    options: dict[str, str] = {}
    with open(ids_file_location, "r") as file:
        readable_line_index = 0
        default_values = True
        line_index = 0
        for line in file:
            line_index += 1
            line = line.strip()
            if len(line) == 0 or line[0] == "#":
                continue
            readable_line_index += 1
            parameters = line.split("\t")
            if len(parameters) > 3:
                raise ValueError(
                    'Invalid format in file {0} on line {1}: "{2}"\n{3}'.format(
                        ids_file_location, line_index, line, "Too many columns"
                    )
                )
            if len(parameters) == 1:
                parameters = parameters[0].split("=")
                if len(parameters) == 1:
                    default_values = False
                    main_ID_dict[parameters[0]] = main_ID_dict.get(
                        parameters[0], [set(), set(), set()]
                    )
                    if "default-flags" not in set(options.keys()):
                        raise ValueError(
                            "Invalid value in file {0} on line {1}: {2}\n{3}".format(
                                ids_file_location,
                                line_index,
                                line,
                                "No default flags used",
                            )
                        )
                    main_ID_dict[parameters[0]][0].update(
                        set(options.get("default-flags", "").split(","))
                    )
                    main_ID_dict[parameters[0]][1].update(
                        set(options.get("default-categories", "").split(","))
                    )
                    continue
                if len(parameters) != 2:
                    raise ValueError(
                        'Invalid format in file {0} on line {1}: "{2}"\n{3}'.format(
                            ids_file_location,
                            line_index,
                            line,
                            "Invalid format of default values",
                        )
                    )
                if (
                    not default_values
                    or readable_line_index > len(VALID_DEFAULTS)
                    or parameters[0] in set(options.keys())
                ):
                    raise ValueError(
                        'Invalid value in file {0} on line {1}: "{2}"\n{3}'.format(
                            ids_file_location,
                            line_index,
                            line,
                            "Too many default options or default options"
                            "are not at the start of the file",
                        )
                    )
                if not check_valid_values(VALID_DEFAULTS, [parameters[0]]):
                    raise ValueError(
                        'Invalid value in file {0} on line {1}: "{2}"\n{3}'.format(
                            ids_file_location,
                            line_index,
                            line,
                            "Invalid default option",
                        )
                    )
                if parameters[0] == "default-flags" and not check_valid_values(
                    VALID_FLAGS, parameters[1].split(",")
                ):
                    raise ValueError(
                        'Invalid value in file {0} on line {1}: "{2}"\n{3}'.format(
                            ids_file_location, line_index, line, "Invalid flag"
                        )
                    )
                if parameters[0] == "default-categories" and not check_valid_values(
                    VALID_CATEGORIES, parameters[1].split(",")
                ):
                    raise ValueError(
                        'Invalid value in file {0} on line {1}: "{2}"\n{3}'.format(
                            ids_file_location, line_index, line, "Invalid category"
                        )
                    )

                options[parameters[0]] = parameters[1]
                continue
            default_values = False
            if (
                len(parameters) >= 2
                and parameters[1] == ""
                and "default-flags" not in set(options.keys())
            ):
                raise ValueError(
                    "Invalid value in file {0} on line {1}: {2}\n{3}".format(
                        ids_file_location, line_index, line, "No default flags used"
                    )
                )
            main_ID_dict[parameters[0]] = main_ID_dict.get(
                parameters[0], [set(), set(), set()]
            )
            if len(parameters) == 2 or len(parameters) > 2 and parameters[2] == "":
                main_ID_dict[parameters[0]][1].update(
                    set(options.get("default-categories", "").split(","))
                )

            for index in range(1, len(parameters)):
                main_ID_dict[parameters[0]][index - 1].update(
                    (set(parameters[index].split(",")))
                )

            if not check_valid_values(
                VALID_FLAGS, list(main_ID_dict[parameters[0]][0])
            ):
                raise ValueError(
                    'Invalid flag in file {0} on line {1}: "{2}"\n{3}'.format(
                        ids_file_location, line_index, line, "Invalid Flag"
                    )
                )

            if not check_valid_values(
                VALID_CATEGORIES, list(main_ID_dict[parameters[0]][1])
            ):
                raise ValueError(
                    'Invalid category in file {0} on line {1}: "{2}"\n{3}'.format(
                        ids_file_location, line_index, line, "Invalid category"
                    )
                )
    return main_ID_dict


def read_IDs_files(files: list[str]) -> dict[str, list[str]] | None:
    if len(files) == 0:
        print("No files were selected")
        return None
    main_ID_dict: ID_List = {}
    for file in files:
        path = Path(file)
        if not path.is_file():
            raise FileExistsError("File {0} doesn't exist".format(file))
        main_ID_dict = read_IDs_file(file, main_ID_dict)
    out_dict: dict[str, list[str]] = {}
    for id in main_ID_dict.keys():
        out_dict[id] = []
        for i in range(0, len(main_ID_dict[id])):
            if "" in main_ID_dict[id][i]:
                main_ID_dict[id][i].remove("")
            option_list = list(main_ID_dict[id][i])
            option_list.sort()
            out_dict[id].append(",".join(option_list))
    return out_dict



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2024 Tomas Votava <thomasvotava@seznam.cz>

import argparse
import xml.etree.ElementTree as ET
from connect_ids import read_IDs_files

POPULAR: ET.Element = ET.Element("kudos")
kudo: ET.Element = ET.SubElement(POPULAR, "kudo")
kudo.text = "GnomeSoftware::popular"

FEATURED: ET.Element = ET.Element("custom")
value: ET.Element = ET.SubElement(
    FEATURED, "value", {"key": "GnomeSoftware::FeatureTile"}
)
value.text = "True"


ID_List = dict[str, list[str]]

XML_Location: str = "org.gnome.App-list.xml"

FLAGS_DICT: dict[str, ET.Element] = {
    "popular": POPULAR,
    "featured": FEATURED,
}

COMPONENT_ATTR: dict[str, str] = {"merge": "append"}


def generate_xml(path: str, complete_list: ID_List) -> None:
    output: str = ""
    output += """<?xml version=\"1.0\" encoding=\"UTF-8\"?>
<!-- SPDX-License-Identifier: LGPL-2.1-or-later -->\n"""
    components: ET.Element = ET.Element("components")
    tree: ET.ElementTree = ET.ElementTree(components)
    for id, flags in complete_list.items():
        if "skip" in set(flags[0].split(",")):
            continue
        component: ET.Element = ET.SubElement(components, "component", COMPONENT_ATTR)
        id_elem: ET.Element = ET.SubElement(component, "id")
        id_elem.text = id
        if len(flags[0]) != 0:
            for flag in flags[0].split(","):
                if flag != "":
                    component.append(FLAGS_DICT[flag])
        if len(flags[1]) != 0:
            categories: ET.Element = ET.SubElement(component, "categories")
            for category in flags[1].split(","):
                category_elem: ET.Element = ET.SubElement(categories, "category")
                category_elem.text = category
    ET.indent(tree)
    with open(path, "w") as file:
        file.write(output)
        file.write(ET.tostring(components, "utf-8").decode("utf-8"))
        return


def generate_xml_from_files(path: str, files: list[str]) -> None:
    ids_dict = read_IDs_files(files)
    if ids_dict is None:
        raise Exception("No IDs were loaded from files")
    generate_xml(path, ids_dict)


def main() -> int:
    parser = argparse.ArgumentParser(
        description="Generates org.gnome.App-list.xml file from given ID files"
    )
    parser.add_argument(
        "file", type=str, nargs="+", help="ID files to create xml file from"
    )
    args = parser.parse_args()
    files = args.file
    generate_xml_from_files(XML_Location, files)
    return 0


if __name__ == "__main__":
    main()
